// card.cpp : Defines the entry point for the console application.
//            

#include "stdafx.h"
#include "casmcard.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define NUMBER_OF_READERS	8
#define NAME_LENGTH			100

int main(int argc, char* argv[])
{
	unsigned char
	szDataOut[262],
	szDataIn[262],
	szAtr[32],
	ARTBuff[128];
	SCARDCONTEXT	hContext;
	SCARDHANDLE		hCard;
	SCARD_READERSTATE	ScardReaderState[NUMBER_OF_READERS];

	DWORD
		lRet,
		dwIndex,
		dwReader,
		dwSelect,
		dwShareMode,
		dwProtocol,
		dwPreferedProtocol,
		dwState,
		dwAtrLength = 32,
		dwLengthOut,
		dwLengthIn,
		dwBytesReturned,
		dwIoctl,
		dwAttribut,dwClass,dwTag,
		dwReadersLength,
		ATRLen;
	LPSTR
		szListReaders;
	LPTSTR
		szFirstReader;

	SCARD_IO_REQUEST
		ScardIoRequest;
	int
		nResponse;

	unsigned char szData[256];

	printf("Smart Card Reader Test Suite V1.0\n");
	printf("Copyright(c) Castles Corporation 2004\n");

	/*------------------------------------------------------------------------------
	Establish a context with the ICC Resource Manager.
	------------------------------------------------------------------------------*/
	lRet = CasEstablishContext(SCARD_SCOPE_USER,NULL,NULL,&hContext);
	if (lRet != SCARD_S_SUCCESS)
	{
		printf("CasEstablishContext fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		return -1;
	}

	/*------------------------------------------------------------------------------
	Allocate the size to read the list of the readers
	Get the list of readers.
	------------------------------------------------------------------------------*/
	lRet = CasListReadersA(hContext,
							NULL,
							NULL,
							&dwReadersLength);

	if (lRet != SCARD_S_SUCCESS)
	{
		if(lRet == SCARD_E_NO_READERS_AVAILABLE)
			printf("CasListReadersA: SCARD_E_NO_READERS_AVAILABLE\n");
		else
			printf("CasListReaders fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}
	szListReaders = (char *) malloc(dwReadersLength);

	lRet = CasListReadersA(hContext,
							NULL,
							(LPTSTR)szListReaders,
							&dwReadersLength);

	if (lRet != SCARD_S_SUCCESS)
	{
		printf("CasListReadersA fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}

	/*------------------------------------------------------------------------------
	Select a reader
	------------------------------------------------------------------------------*/
	ZeroMemory((LPVOID)ScardReaderState, sizeof(ScardReaderState));

	dwIndex = 0;
	dwReader = 0;
	szFirstReader = (LPTSTR)szListReaders;
	while (*(szFirstReader) != '\0')
	{
		dwShareMode = SCARD_SHARE_DIRECT;
		dwPreferedProtocol = 0;
		lRet = CasConnect(hContext, 
							(LPCTSTR)szFirstReader, 
							dwShareMode, 
							dwPreferedProtocol, 
							&hCard, 
							&dwProtocol);
		if (lRet != SCARD_S_SUCCESS)
		{
			printf("CasConnect fail! ErrorCode = %08X\n",lRet);
			printf("Press <ENTER> key to terminate!\n");
			nResponse = getchar();
			lRet = CasReleaseContext(hContext);
			return -1;
		}
		else
		{
			if(dwReader==0)
				printf("\t-------------------------\n");
			printf("\t%d:<%s>\n",(dwReader + 1),szFirstReader);
			ScardReaderState[dwReader].szReader = (LPCTSTR) szFirstReader;
			ScardReaderState[dwReader].dwCurrentState = SCARD_STATE_UNAWARE;
			dwReader++;
		}
		lRet = CasDisconnect(hCard,SCARD_UNPOWER_CARD);
		dwIndex += strlen(szFirstReader) + 1;
		szFirstReader = (LPTSTR)(szListReaders + dwIndex);
	}
	if(dwReader >0)
	{
		printf("\t-------------------------\n");
		if(dwReader>1)
		{
			printf("Select a reader number (1 to %d): ",dwReader);
			scanf("%d",&dwSelect);
			if((dwSelect >dwReader+1) || (dwSelect <=0))
			{
				printf("Press <ENTER> key to terminate!\n");
				nResponse = getchar();
				lRet = CasReleaseContext(hContext);
				return -1;
			}
			dwSelect--;
		}
		else
			dwSelect = 0;
	}
	else
	{
		printf("No reader available !");
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}

	printf("=============================\n");
	printf("Part A: Card Connection Testing\n");
	printf("=============================\n");
	//------------------------------------------------------------------------------
	//Establish a context with the card
	//------------------------------------------------------------------------------
	dwShareMode = SCARD_SHARE_SHARED;
	dwPreferedProtocol = SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1;
	lRet = CasConnect(hContext, (LPCTSTR)ScardReaderState[dwSelect].szReader, dwShareMode, dwPreferedProtocol, &hCard, &dwProtocol);
	if (lRet != SCARD_S_SUCCESS)
	{
		printf("CasConnect fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}
	else
	{
		printf("\tCard connection success!\n\n");
	}

	// ------------------------------------------------------------------------------
	// Communication with Card Chcek within the specified reader
	// ------------------------------------------------------------------------------
	printf("=============================\n");
	printf("Part B: Card R/W Testing\n");
	printf("=============================\n");
	printf("1. CPU Card APDU Test\n");
	printf("2. Read Memory Card\n");
	printf("3. Get ART\n");
	//------------------------------------------------------------------------------

Card_R_W_Test:

	dwSelect = 0;
	scanf("%d",&dwSelect);
	
	if(dwSelect == 1) //APDU Test
	{
		printf("\t1.APDU Command Test (Read Binary)");

		dwLengthOut = 256;
		dwLengthIn = 5;
		memset(szDataIn,0,sizeof(szDataIn));
		memset(szDataOut,0,sizeof(szDataOut));
		
		memcpy(szDataIn,"\x00\xB0\x00\x00\x10",dwLengthIn);
                    // APDU Command "Read Binary"
					//	00 -> CLA Byte
					//	B0 -> INS Byte
					//	00 -> P1  Byte
					//	00 -> P2  Byte
					//	10 -> Le  Byte
					

		ScardIoRequest.dwProtocol = dwProtocol;
		ScardIoRequest.cbPciLength = sizeof(ScardIoRequest);

		lRet = CasTransmit(hCard,
					&ScardIoRequest,
					szDataIn,
					dwLengthIn,
					NULL,
					szDataOut,
					&dwLengthOut);

		if(lRet == 0)
		{
			int i;
			printf(" Success!\n");
			printf("\nRead Binary receive Len:%d ,Stateword:%02X %02X ,Data :\n",dwLengthOut,szDataOut[dwLengthOut-2],szDataOut[dwLengthOut-1]);
			for(i=0;i<dwLengthOut;i++)
			{
				printf("%02X ",szDataOut[i]);	
			}
			printf("\n");
		}
		else
		{
			printf(" fail! ErrorCode = 0x%08X\n",lRet);
			lRet = CasReleaseContext(hContext);
			system("pause");
			return -1;
		}

	}
	else if(dwSelect == 2) //Read memory card
	{
		printf("\t2.SLE4442_Read_Main_Memory");

		int StartAddr;
		DWORD recvlength;
		BYTE recvbuffer[512];

		StartAddr = 0;
		recvlength = 16;
		lRet = SLE4442_Read_Main_Memory(hCard,
										 recvbuffer,
										 StartAddr,
										 &recvlength);

		if(lRet == SCARD_S_SUCCESS)
		{
			int i;
			printf(" Success!\n");			
			printf("\nRead Main Memory Receive Len:%d  ,Data :\n",recvlength);
			for(i=0;i<recvlength;i++)
			{
				printf("%02X ",recvbuffer[i]);	
			}
			printf("\n");
		}
		else
		{
			printf(" fail! ErrorCode = 0x%08X\n",lRet);
			lRet = CasReleaseContext(hContext);
			system("pause");
			return -1;
		}
	}
	else if(dwSelect == 3) //Get  ATR
	{
		memset(ARTBuff,0,sizeof(ARTBuff) );
		ATRLen = sizeof(ARTBuff);

		lRet = CasGetAttrib( hCard,
			SCARD_ATTR_ATR_STRING,
			ARTBuff,
			&ATRLen);

		if(lRet == 0)
		{
			int i;
			printf(" Success!\n");
			printf("\nATR receive Len:%d ,ATR Data :\n",ATRLen);
			for(i=0;i<ATRLen;i++)
			{
				printf("%02X ",ARTBuff[i]);	
			}
			printf("\n");
		}
		else
		{
			printf(" fail! ErrorCode = 0x%08X\n",lRet);
			lRet = CasReleaseContext(hContext);
			system("pause");
			return -1;
		}

	}
	else
	{
		goto Card_R_W_Test;
	}
	


	/*------------------------------------------------------------------------------
	Terminates the opened connection with the card.
	------------------------------------------------------------------------------*/
	lRet = CasDisconnect(hCard,SCARD_LEAVE_CARD);
	if (lRet != SCARD_S_SUCCESS)
		printf("CasDisconnect: %Xh\n",lRet);
	
	/*------------------------------------------------------------------------------
	Close the establish context with the ICC Resource Manager.
	------------------------------------------------------------------------------*/
	lRet = CasReleaseContext(hContext);
	if (lRet != SCARD_S_SUCCESS)
		printf("CasReleaseContext: %Xh\n",lRet);

	//printf("\nPress <ENTER> key to terminate!\n");
	//nResponse = getchar();
	system("pause");
	return 0;
}
