#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "casmcard.h"
#define NUMBER_OF_READERS	8
#define NAME_LENGTH			100

void main(void)
{
	unsigned char
		szDataOut[262],
		szDataIn[262],
		szAtr[32];

	SCARDCONTEXT
		hContext;
	SCARDHANDLE
		hCard;
	SCARD_READERSTATE 
		ScardReaderState[NUMBER_OF_READERS];
	DWORD
		lRet,
		dwIndex,
		dwReader,
		dwSelect,
		dwShareMode,
		dwProtocol,
		dwPreferedProtocol,
		dwState,
		dwAtrLength = 32,
		dwLengthOut,
		dwLengthIn,
		dwBytesReturned,
		dwIoctl,
		dwAttribut,dwClass,dwTag,
		dwReadersLength;
	LPSTR
		szListReaders;
	LPTSTR
		szFirstReader;
	int
		i, nResponse;

	SCARD_IO_REQUEST
		ScardIoRequest;

	BYTE key[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
	const BYTE initvalueblock[16] = {0x64,0x00,0x00,0x00,
									  0x9b,0xff,0xff,0xff,
					   				  0x64,0x00,0x00,0x00,
									  0x05,0xfa,0x05,0xfa};
	BYTE Mifare_SN[10];
	DWORD Mifare_Len;
	BYTE block[16];

	printf("EZ710 Smart Card Reader Test Suite V1.0\n");
	printf("Copyright(c) Castles Corporation 2012\n");

	/*------------------------------------------------------------------------------
	Establish a context with the ICC Resource Manager.
	------------------------------------------------------------------------------*/
	lRet = CasEstablishContext(SCARD_SCOPE_USER,NULL,NULL,&hContext);
	if (lRet != SCARD_S_SUCCESS)
	{
		printf("SCardEstablishContext fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		return;
	}

	/*------------------------------------------------------------------------------
	Allocate the size to read the list of the readers
	Get the list of readers.
	------------------------------------------------------------------------------*/
	lRet = CasListReaders(hContext,
							NULL,
							NULL,
							&dwReadersLength);

	if (lRet != SCARD_S_SUCCESS)
	{
		if(lRet == SCARD_E_READER_UNAVAILABLE)
			printf("SCardListReaders: SCARD_E_NO_READERS_AVAILABLE\n");
		else
			printf("SCardListReaders fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return;
	}
	szListReaders = malloc(dwReadersLength);

	lRet = CasListReaders(hContext,
							NULL,
							(LPTSTR)szListReaders,
							&dwReadersLength);

	if (lRet != SCARD_S_SUCCESS)
	{
		printf("SCardListReader fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return;
	}

	/*------------------------------------------------------------------------------
	Select a reader
	------------------------------------------------------------------------------*/
	ZeroMemory((LPVOID)ScardReaderState, sizeof(ScardReaderState));

	dwIndex = 0;
	dwReader = 0;
	szFirstReader = (LPTSTR)szListReaders;

	while (*(szFirstReader) != '\0')
	{
		if(strcmp("CASTLES EZ710BU_CL 0",szFirstReader) == 0)
		{
		
			dwShareMode = SCARD_SHARE_SHARED;
			dwPreferedProtocol = SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1 ;
			
			lRet = CasConnect(hContext, 
								(LPCTSTR)szFirstReader, 
								dwShareMode, 
								dwPreferedProtocol, 
								&hCard, 
								&dwProtocol);
			if (lRet != SCARD_S_SUCCESS)
			{
				printf("SCardConnect fail! ErrorCode = %08X\n",lRet);
				printf("Press <ENTER> key to terminate!\n");
				nResponse = getchar();
				lRet = CasReleaseContext(hContext);
				return;
			}
			else
			{
				printf("Please select type of card to test :\n");
				printf("1. Mifare\n");
				printf("2. Type A\n");
				printf(">> ");
				nResponse = getchar();
				
				if(nResponse == '1')
				{
					//ULONG i;
					printf("SCardConnect success\n");

					//Sleep(1000);
					lRet = CasCLGetSN(hCard,Mifare_SN,10,&Mifare_Len);
					printf("get sn = 0x%X\n",lRet);
					for(i = 0 ; i < Mifare_Len ; i++)
						printf("%02X ",Mifare_SN[i]);
					printf("\n");

					printf("Key = ");
					for(i = 0 ; i < 6 ; i++)
						printf("%02X ",key[i]);
					printf("\n");

					lRet = CasMifareAuth(hCard,0x60,0x05,key,Mifare_SN);
					printf("auth = 0x%X\n",lRet);

					lRet = CasMifareReadBlock(hCard,0x05,block);
					printf("read block = 0x%X\n",lRet);
					for(i = 0 ; i < 16 ; i++)
						printf("%x ",block[i]);
					printf("\n");

					lRet = CasMifareWriteBlock(hCard,0x05,(BYTE *)initvalueblock);
					printf("write block = 0x%X\n",lRet);
					lRet = CasMifareReadBlock(hCard,0x05,block);
					printf("read block = 0x%X\n",lRet);
					for(i = 0 ; i < 16 ; i++)
						printf("%x ",block[i]);
					printf("\n");
				
					
					lRet = CasMifareIncrement(hCard,0x05,100);
					printf("increment = 0x%X\n",lRet);
					
					lRet = CasMifareTransfer(hCard,0x05);
					printf("transfer = 0x%X\n",lRet);

					lRet = CasMifareDecrement(hCard,0x05,50);
					printf("Decrement = 0x%X\n",lRet);
									
					lRet = CasMifareTransfer(hCard,0x05);
					printf("transfer = 0x%X\n",lRet);

					lRet = CasMifareReadBlock(hCard,0x05,block);
					printf("read block = 0x%X\n",lRet);
					for(i = 0 ; i < 16 ; i++)
						printf("%x ",block[i]);
					printf("\n");
				}
				else if(nResponse == '2')
				{
					printf("\t1. Select EF(3F00)");
					dwLengthOut = 200;
					dwLengthIn = 7;
					memcpy(szDataIn,"\x00\xA4\x02\x00\x02\x3F\x00",dwLengthIn);

					//	00 -> CLA Byte
					//	A4 -> INS Byte
					//	02 -> P1  Byte
					//	00 -> P2  Byte
					//	02 -> EF ID Length
					//	00B0 -> EF ID
	
					ScardIoRequest.dwProtocol = dwProtocol;
					ScardIoRequest.cbPciLength = sizeof(ScardIoRequest);
					lRet = CasTransmit(hCard,&ScardIoRequest,szDataIn,dwLengthIn,NULL,szDataOut,&dwLengthOut);
					if (lRet != SCARD_S_SUCCESS)
					{
						printf("CasTransmit fail! ErrorCode = %08X\n",lRet);
						printf("Press <ENTER> key to terminate!\n");
						nResponse = getchar();
						lRet = CasDisconnect(hCard,SCARD_UNPOWER_CARD);
						lRet = CasReleaseContext(hContext);
						return;
					}
					else
					{
						printf("\n Send:");
						for(i = 0 ; i < dwLengthIn ; i++)
							printf("%02X ",szDataIn[i]);
						printf("\n Recv:");
						for(i = 0 ; i < dwLengthOut ; i++)
							printf("%02X ",szDataOut[i]);
						printf("\n");
					}

				if ((dwLengthOut == 2) && (memcmp(szDataOut,"\x90\x00",2)==0))
					printf("\tpassed\n");

					
				}
				
				lRet = CasDisconnect(hCard,SCARD_UNPOWER_CARD);
			}
			
		
		}
		dwIndex += strlen(szFirstReader) + 1;
		szFirstReader = (LPTSTR)(szListReaders + dwIndex);
	}
	system("pause");
	return;

}
