#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <winscard.h>
#include "casmcard.h"
#define NUMBER_OF_READERS	8
#define NAME_LENGTH			100


int main(int argc, char* argv[])
{
	unsigned char
	szDataOut[262],
	szDataIn[262],
	szAtr[32];
	SCARDCONTEXT	hContext;
	SCARDHANDLE		hCard;
	SCARD_READERSTATE	ScardReaderState[NUMBER_OF_READERS];

	DWORD
		lRet,
		dwIndex,
		dwReader,
		dwSelect,
		dwShareMode,
		dwProtocol,
		dwPreferedProtocol,
		dwState,
		dwAtrLength = 32,
		dwLengthOut,
		dwLengthIn,
		dwBytesReturned,
		dwIoctl,
		dwAttribut,dwClass,dwTag,
		dwReadersLength;
	LPSTR
		szListReaders;
	LPTSTR
		szFirstReader;

	SCARD_IO_REQUEST
		ScardIoRequest;
	int
		nResponse;

	unsigned char szData[256];

	printf("Smart Card Reader Test Suite V1.0\n");
	printf("Copyright(c) Castles Corporation 2004\n");
	
	/*------------------------------------------------------------------------------
	Establish a context with the ICC Resource Manager.
	------------------------------------------------------------------------------*/
	lRet = CasEstablishContext(SCARD_SCOPE_USER,NULL,NULL,&hContext);
	if (lRet != SCARD_S_SUCCESS)
	{
		printf("CasEstablishContext fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		return -1;
	}

	/*------------------------------------------------------------------------------
	Allocate the size to read the list of the readers
	Get the list of readers.
	------------------------------------------------------------------------------*/
	lRet = CasListReaders(hContext,
							NULL,
							NULL,
							&dwReadersLength);

	if (lRet != SCARD_S_SUCCESS)
	{
		if(lRet == SCARD_E_READER_UNAVAILABLE)
			printf("CasListReadersA: SCARD_E_NO_READERS_AVAILABLE\n");
		else
			printf("CasListReaders fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}
	szListReaders = (char *) malloc(dwReadersLength);

	lRet = CasListReaders(hContext,
							NULL,
							(LPTSTR)szListReaders,
							&dwReadersLength);

	if (lRet != SCARD_S_SUCCESS)
	{
		printf("CasListReadersA fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}

	/*------------------------------------------------------------------------------
	Select a reader
	------------------------------------------------------------------------------*/
	//ZeroMemory((LPVOID)ScardReaderState, sizeof(ScardReaderState));
	memset((LPVOID)ScardReaderState,0, sizeof(ScardReaderState));
	dwIndex = 0;
	dwReader = 0;
	szFirstReader = (LPTSTR)szListReaders;
	while (*(szFirstReader) != '\0')
	{
		dwShareMode = SCARD_SHARE_DIRECT;
		dwPreferedProtocol = 0;
		lRet = CasConnect(hContext, 
							(char*)szFirstReader, 
							dwShareMode, 
							dwPreferedProtocol, 
							&hCard, 
							&dwProtocol);
		if (lRet != SCARD_S_SUCCESS)
		{
			printf("CasConnect fail! ErrorCode = %08X\n",lRet);
			printf("Press <ENTER> key to terminate!\n");
			nResponse = getchar();
			lRet = CasReleaseContext(hContext);
			return -1;
		}
		else
		{
			if(dwReader==0)
				printf("\t-------------------------\n");
			printf("\t%d:<%s>\n",(dwReader + 1),szFirstReader);
			ScardReaderState[dwReader].szReader = (char*) szFirstReader;
			ScardReaderState[dwReader].dwCurrentState = SCARD_STATE_UNAWARE;
			dwReader++;
		}
		lRet = CasDisconnect(hCard,SCARD_UNPOWER_CARD);
		dwIndex += strlen(szFirstReader) + 1;
		szFirstReader = (LPTSTR)(szListReaders + dwIndex);
	}
	if(dwReader >0)
	{
		printf("\t-------------------------\n");
		if(dwReader>1)
		{
			printf("Select a reader number (1 to %d): ",dwReader);
			scanf("%d",&dwSelect);
			if((dwSelect >dwReader+1) || (dwSelect <=0))
			{
				printf("Press <ENTER> key to terminate!\n");
				nResponse = getchar();
				lRet = CasReleaseContext(hContext);
				return -1;
			}
			dwSelect--;
		}
		else
			dwSelect = 0;
	}
	else
	{
		printf("No reader available !");
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}

	printf("=============================\n");
	printf("Part A: Card Connection Testing\n");
	printf("=============================\n");
	//------------------------------------------------------------------------------
	//Establish a context with the card
	//------------------------------------------------------------------------------
	dwShareMode = SCARD_SHARE_SHARED;
	dwPreferedProtocol = SCARD_PROTOCOL_T0 | SCARD_PROTOCOL_T1;
	lRet = CasConnect(hContext, (char*)ScardReaderState[dwSelect].szReader, dwShareMode, dwPreferedProtocol, &hCard, &dwProtocol);
	if (lRet != SCARD_S_SUCCESS)
	{
		printf("CasConnect fail! ErrorCode = %08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}

	// ------------------------------------------------------------------------------
	// Communication with Card Chcek within the specified reader
	// ------------------------------------------------------------------------------
	printf("=============================\n");
	printf("Part B: Card R/W Testing\n");
	printf("=============================\n");
	//------------------------------------------------------------------------------

	printf("\t1. SLE4442_Read_Main_Memory");

	int StartAddr;
	DWORD recvlength;
	BYTE recvbuffer[512];

	StartAddr = 0;
	recvlength = 16;
	lRet = SLE4442_Read_Main_Memory(hCard,
		                             recvbuffer,
                                     StartAddr,
                                     &recvlength);

	if(lRet == SCARD_S_SUCCESS)
	{
		printf(" Success!\n");
	}
	else
	{
		printf(" fail! ErrorCode = 0x%08X\n",lRet);
		printf("Press <ENTER> key to terminate!\n");
		nResponse = getchar();
		lRet = CasReleaseContext(hContext);
		return -1;
	}


	/*------------------------------------------------------------------------------
	Terminates the opened connection with the card.
	------------------------------------------------------------------------------*/
	lRet = CasDisconnect(hCard,SCARD_LEAVE_CARD);
	if (lRet != SCARD_S_SUCCESS)
		printf("CasDisconnect: %Xh\n",lRet);
	
	/*------------------------------------------------------------------------------
	Close the establish context with the ICC Resource Manager.
	------------------------------------------------------------------------------*/
	lRet = CasReleaseContext(hContext);
	if (lRet != SCARD_S_SUCCESS)
		printf("CasReleaseContext: %Xh\n",lRet);

	printf("\nPress <ENTER> key to terminate!\n");
	nResponse = getchar();
	return 0;
}
